#include <windows.h>
#include "snes9x/snes9x.h"
#include "snes9x/memmap.h"
#include "gui.h"
#include "menu.h"
#include "global.h"

int gint_primary = -1;
int gint_port = -1;

#ifdef UNDER_CE

#include "mapledev.h"
#include "Perstore.h"

static IFlashDevice *pIFlash[4][2] = { NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL };
static int FlashFree[4][2] = { -1, -1, -1, -1, -1, -1, -1, -1 };
static IFlashFile *pIFile = NULL;
static int sz;

int GetSRAMSize(void) { 
	sz = Memory.SRAMSize ?
	       (1 << (Memory.SRAMSize + 3)) * 128 : 0;
	return (sz / 512) + 1; 
}

int GetFlashStatus(int port, int dev) { 
	if(pIFlash[port][dev]) return FlashFree[port][dev];
	else return -1;
}

void GetCurrentFlashPort(int* p, int* dev) {
	*p = gint_primary;
	*dev = gint_port;
}

void S9xSaveSRAM(const char *filename, int size, const char *data) {
	char str[256];
	HRESULT hr;

	S9xMessage(0, 0, "S9xSaveSRAM called");
	sprintf(str, "Saving VMS File: %s", filename);
	S9xMessage(0, 0, str);

	hr = pIFlash[gint_primary][gint_port]->OpenFlashFileByName(&pIFile, gp_SRAMFile);
	
	hr = pIFile->Write(0, size, (BYTE *)data);
	if(FAILED(hr)) {
		S9xMessage(0, 0, "IFlashFile::Write failed");
		return;
	}
	hr = pIFile->Flush();
	if(FAILED(hr)) {
		S9xMessage(0, 0, "IFlashFile::Flush failed");
		return;
	}

	pIFile->Release();
	pIFile = NULL;
	
	S9xMessage(0, 0, "Save OK");
}

BOOL S9xLoadSRAM(const char *filename, int size, char* data) {
	char str[256];
	HRESULT hr;

	S9xMessage(0, 0, "S9xLoadSRAM called");
	sprintf(str, "Opening VMS File: %s", filename);
	S9xMessage(0, 0, str);
	
	hr = pIFlash[gint_primary][gint_port]->OpenFlashFileByName(&pIFile, (char *)filename);
	switch(hr) {
		case VMS_OK:
			S9xMessage(0, 0, "VMS File found.");
			hr = pIFile->Read(0, size, (BYTE *)data);
			if(FAILED(hr)) {
				S9xMessage(0, 0, "IFlashFile::Read failed");
				return TRUE;
			}
			S9xMessage(0, 0, "Load OK");
			return TRUE;
		case E_INVALIDARG:
			S9xMessage(0, 0, "Unexpected Error: Filename = NULL");
			break;
		case E_ACCESSDENIED:
			S9xMessage(0, 0, "File not Found");
			break;
		case E_OUTOFMEMORY:
			S9xMessage(0, 0, "Out of Memory");
			break;
		default:
			S9xMessage(0, 0, "General Error");
	}

	pIFile->Release();
	pIFile = NULL;

	sz = size;
	return FALSE;
}

bool found;

BOOL EnumFlashFilesCallback(IFlashDevice*, FSFILEID, LPCFSFILEDESC lpcfsfiledesc, void* fn) {
	TCHAR str[256];
	mbstowcs(str, (char*)fn, strlen((char*)fn));

//	const TCHAR* txt3[] = { _T(" "), str, _T(" ") };
//	DoMenu(3, txt3, 0, NULL, NULL, widthFIT, heightFIT, 0, false);

	if(!strcmp(lpcfsfiledesc->szFileName, (char*)fn)) {
		found = true;

//		const TCHAR* txt4[] = { _T(" "), _T("Found"), _T(" ") };
//		DoMenu(3, txt4, 0, NULL, NULL, widthFIT, heightFIT, 0, false);
	}

	if(!found) return true;
	return false;
}

bool FlashFindFile(const char *filename) {
	found = false;

	HRESULT hr = pIFlash[gint_primary][gint_port]->EnumFlashFiles(EnumFlashFilesCallback, (void*)filename);

	if(hr != S_OK) {
		const TCHAR* txt3[] = { _T(" "), ErrorMsg(hr), _T(" ") };
		DoMenu(3, txt3, 0, NULL, NULL, widthFIT, heightFIT, 0, false);
	}

	return found;
}

BOOL FlashCreateFile(const char *filename, const char *desc, int size) {
	char str[256];
	FSFILEDESC fsfd;

	memset(&fsfd, 0, sizeof(fsfd));

	fsfd.dwSize = sizeof(FSFILEDESC);
	fsfd.dwFlags = FSFD_CREATE_FILE;
	fsfd.dwBytesRequired = sz;
	strcpy(fsfd.szFileName, filename);
	strcpy(fsfd.szVMSComment, desc);
	strcpy(fsfd.szGameName, "Sintendo");
	strcpy(fsfd.szBootROMComment, "Sintendo");
	fsfd.bStatus = FS_STATUS_DATA_FILE;
	fsfd.bCopy = FS_COPY_ENABLED;

	S9xMessage(0, 0, "Creating file...");

	HRESULT hr = pIFlash[gint_primary][gint_port]->CreateFlashFile(&pIFile, &fsfd);
	if(FAILED(hr)) {
		sprintf(str, "HRESULT = %i %X", hr, hr);
		S9xMessage(0, 0, "CreateFlashFile failed");
		S9xMessage(0, 0, str);
		return FALSE;
	}

	S9xMessage(0, 0, "CreateFlashFile OK");

	return TRUE;
}

BOOL PASCAL EnumMapleDevicesCallback(LPCMAPLEDEVICEINSTANCE pmdi,
									 LPVOID context) {

	HRESULT hr;
	IUnknown *pIUnknown = NULL;
	IFlashDevice* pIFlashTemp = NULL;

#ifdef DEBUG_FLASH
	MessageBox(_T("EnumMapleDevicesCallback()"), false);
#endif

	hr = MapleCreateDevice(&pmdi->guidDevice, &pIUnknown);
	if(FAILED(hr)) {
		MessageBox(_T("MapleCreateDevice Failed"));
		MessageBox(ErrorMsg(hr));
		return TRUE;
	}

	hr = pIUnknown->QueryInterface(_uuidof(IFlashDevice), (void**)&pIFlashTemp);
	pIUnknown->Release();
	if(FAILED(hr)) {
		MessageBox(_T("QueryInterface Failed"));
		MessageBox(ErrorMsg(hr));
		return TRUE;
	}

#ifdef DEBUG_FLASH
	MessageBox(_T("IFlashDevice found"), false);
#endif

	// Begin Query Flash Device	
	FSDEVICEDESC fsdevicedesc;

	memset(&fsdevicedesc, 0, sizeof(fsdevicedesc));
	fsdevicedesc.dwSize = sizeof(FSDEVICEDESC);
	fsdevicedesc.dwFlags = FSDD_PORT_NUMBER | FSDD_DEVICE_NUMBER | FSDD_FREE_BLOCKS;
	hr = pIFlashTemp->GetDeviceDesc(&fsdevicedesc);

	if(FAILED(hr)) {
		MessageBox(_T("GetDeviceDesc failed"));
		MessageBox(ErrorMsg(hr));
		return TRUE;
	}

#ifdef DEGUB_FLASH
	TCHAR str[256];
	_stprintf(str, _T("FlashDevice Info: PORT: %i DEVICE: %i"), fsdevicedesc.dwPortNumber,
		fsdevicedesc.dwDeviceNumber);
	MessageBox(str, false);
//	sprintf(str, "%i Free Blocks", fsdevicedesc.dwFreeBlocks);
//	S9xMessage(0, 0, str);
#endif

	pIFlash[fsdevicedesc.dwPortNumber][fsdevicedesc.dwDeviceNumber-1] = pIFlashTemp;
	FlashFree[fsdevicedesc.dwPortNumber][fsdevicedesc.dwDeviceNumber-1] = fsdevicedesc.dwFreeBlocks;

	return TRUE;
}

#endif

void FlashInit(void) {
#ifdef UNDER_CE
	HRESULT hr;

#ifdef DEBUG_FLASH
	MessageBox(_T("MapleEnumerateDevices()"), false);
#endif

	hr = MapleEnumerateDevices(MDT_STORAGE, EnumMapleDevicesCallback, NULL, 0);

	if(FAILED(hr)) {
		MessageBox(_T("MapleEnumerateDevices failed"));
		S9xMessage(0, 0, "MapleEnumerateDevices failed");
	}

	if(!gb_insaveloadmenu) DoSaveLoadMenu(true);
#endif
}

void FlashDeInit(void) {
#ifdef UNDER_CE

//	MessageBox(_T("Flash DeInit()"));

	if(pIFile != NULL) {
		pIFile->Flush();
		pIFile->Release();
		pIFile = NULL;
	}

	for(int i = 0; i < 4; i++) {
		for(int j = 0; j < 2; j++) {
			if(pIFlash[i][j] != NULL) {
				pIFlash[i][j]->Release();
				pIFlash[i][j] = NULL;
				FlashFree[i][j] = -1;
			}
		}
	}
#endif
}

#ifndef UNDER_CE

void S9xSaveSRAM(const char *filename, int size, const char *data) { return; }
BOOL S9xLoadSRAM(const char *filename, int size, char* data) { return FALSE; }
BOOL FlashCreateFile(const char *filename, const char *desc, int size) { return FALSE; }
int GetFlashStatus(int port, int dev) { 
	if(port == 0 && dev == 1) return 15;
	if(port == 2 && dev == 0) return 87;
	return -1;
}
void GetCurrentFlashPort(int* p, int* dev) { *p = 0, *dev = 0; }
int GetSRAMSize(void) { return 0; }

#endif