#include "Direct3D.h"
#include "global.h"
#include <stdio.h>
#include "snes9x/snes9x.h"

// Return the Direct3D object if successful, also sets the global Direct3D object
// Returns NULL on failure
LPDIRECT3D3 InitDirect3D() {
	HRESULT hr;
	char str[256];

	hr = gpdd_directdraw->QueryInterface(IID_IDirect3D3, (LPVOID*)&gd3d_direct3d);
	if(hr != DD_OK) {
		sprintf(str, "Failed to Query Direct3D Object. %s", ErrorMsg(hr));
		S9xMessage(0, 0, str);
		return NULL;
	}

	return gd3d_direct3d;
}

LPDIRECT3DVIEWPORT3 vp = NULL;
D3DVIEWPORT2 viewData;   // Viewport Data Structure to fill

LPDIRECT3DDEVICE3 GetDeviceFromSurface(LPDIRECTDRAWSURFACE4 surf, LPDIRECT3DVIEWPORT3 *vp2) {
	char				str[256];
	HRESULT				hr;
	LPDIRECT3DDEVICE3	device;

#if defined UNDER_CE
#define DEVICE_TYPE IID_IDirect3DHALDevice
#else
//#define DEVICE_TYPE IID_IDirect3DTnLHalDevice
//#define DEVICE_TYPE IID_IDirect3DMMXDevice
#define DEVICE_TYPE IID_IDirect3DRGBDevice
#endif

	hr = gd3d_direct3d->CreateDevice(DEVICE_TYPE, surf, &device, NULL);
	if(hr != DD_OK) {
		sprintf(str, "Failed to Create Direct3D Device. %s", ErrorMsg(hr));
		S9xMessage(0, 0, str);
	}
//	sprintf(str, "Device created: dev = %X", device);
//	S9xMessage(0, 0, str);

    hr = gd3d_direct3d->CreateViewport(&vp, NULL);
	if(hr != DD_OK) {
		sprintf(str, "Failed to Create Viewport. %s", ErrorMsg(hr));
		S9xMessage(0, 0, str);
	}

	hr = device->AddViewport(vp);
	if(hr != DD_OK) {
		sprintf(str, "Failed to Add Viewport. %s", ErrorMsg(hr));
		S9xMessage(0, 0, str);
	}

    // Fill the D3DVIEWPORT data structure
    memset(&viewData, 0, sizeof(D3DVIEWPORT2));
    viewData.dwSize   = sizeof(D3DVIEWPORT2);
    viewData.dwX      = viewData.dwY = 0;
    viewData.dwWidth  = 640;
    viewData.dwHeight = 480;

    viewData.dvClipWidth  = 640;
    viewData.dvClipHeight = 480;

    viewData.dvClipX = 0;
    viewData.dvClipY = 0;

    viewData.dvMaxZ = 1.0F;
    viewData.dvMinZ = 0.0F;

	hr = vp->SetViewport2(&viewData);
	if(hr != DD_OK) {
		sprintf(str, "Failed to Set Viewport. %s", ErrorMsg(hr));
		S9xMessage(0, 0, str);
	}

	hr = device->SetCurrentViewport(vp);
	if(hr != DD_OK) {
		sprintf(str, "Failed to Set Current Viewport. %s", ErrorMsg(hr));
		S9xMessage(0, 0, str);
	}

	return device;
}


void FillGradientRect(LPDIRECT3DDEVICE3 dev, int x1, int y1, int x2, int y2,
					 D3DCOLOR tl, D3DCOLOR tr, D3DCOLOR bl, D3DCOLOR br) {

//	FILE *f = fopen("fillgradientrect.trace.txt", "wt");
//	if(!f) S9xMessage(0, 0, "Failed to open Trace file");
	
	D3DTLVERTEX rgverts[5] = {
		{ 0, 0, 0, 1.0f, 0x00000000, 0, 0.0f, 0.0f },
		{ 0, 0, 0, 1.0f, 0x00000000, 0, 0.0f, 0.0f },
		{ 0, 0, 0, 1.0f, 0x00000000, 0, 0.0f, 0.0f },
		{ 0, 0, 0, 1.0f, 0x00000000, 0, 0.0f, 0.0f },
		{ 0, 0, 0, 1.0f, 0x00000000, 0, 0.0f, 0.0f }
	};
	HRESULT hr;

	dev->BeginScene();

//	tl = tr = bl = br = 0xFFFFFFFF;

	rgverts[0].color = tl;
	rgverts[0].sx = (D3DVALUE)x1;
	rgverts[0].sy = (D3DVALUE)y1;
	rgverts[0].sz = 1.0f;
	rgverts[0].specular = D3DRGBA(0, 0, 0, 0);
	rgverts[0].rhw = 1.0f;
	rgverts[0].tu = 0.0f;
	rgverts[0].tv = 0.0f;

	rgverts[1].color = tr;
	rgverts[1].sx = (D3DVALUE)x2;
	rgverts[1].sy = (D3DVALUE)y1;
	rgverts[1].sz = 1.0f;
	rgverts[1].specular = D3DRGBA(0, 0, 0, 0);
	rgverts[1].rhw = 1.0f;
	rgverts[1].tu = 0.0f;
	rgverts[1].tv = 0.0f;

	rgverts[2].color = bl;
	rgverts[2].sx = (D3DVALUE)x1;
	rgverts[2].sy = (D3DVALUE)y2;
	rgverts[2].sz = 1.0f;
	rgverts[2].specular = D3DRGBA(0, 0, 0, 0);
	rgverts[2].rhw = 1.0f;
	rgverts[2].tu = 0.0f;
	rgverts[2].tv = 0.0f;

   	rgverts[3].color = br;
	rgverts[3].sx = (D3DVALUE)x2;
	rgverts[3].sy = (D3DVALUE)y2;
	rgverts[3].sz = 1.0f;
	rgverts[3].specular = D3DRGBA(0, 0, 0, 0);
	rgverts[3].rhw = 1.0f;
	rgverts[3].tu = 0.0f;
	rgverts[3].tv = 0.0f;

    // Alpha blts are always done on top of the screen
//	dev->SetRenderState(D3DRENDERSTATE_ZFUNC, D3DCMP_ALWAYS );

	// Set the alpha blending state
//	dev->SetRenderState(D3DRENDERSTATE_SRCBLEND,  D3DBLEND_SRCALPHA);
//	dev->SetRenderState(D3DRENDERSTATE_DESTBLEND, D3DBLEND_INVSRCALPHA);
//	dev->SetRenderState(D3DRENDERSTATE_BLENDENABLE, TRUE);
	dev->SetRenderState(D3DRENDERSTATE_ALPHABLENDENABLE, TRUE );
	dev->SetRenderState(D3DRENDERSTATE_SRCBLEND, D3DBLEND_BOTHSRCALPHA);

	dev->SetRenderState(D3DRENDERSTATE_TEXTUREHANDLE, NULL);


    // Blow away these states - the app should restore these if it cares.
//    dev->SetRenderState(D3DRENDERSTATE_SPECULARENABLE,   FALSE);
//    dev->SetRenderState(D3DRENDERSTATE_CULLMODE,         D3DCULL_CCW);
//    dev->SetRenderState(D3DRENDERSTATE_ZENABLE, TRUE);
//    dev->SetRenderState(D3DRENDERSTATE_ZWRITEENABLE, TRUE);

    // Draw the surface to the screen via DrawPrimitive.
	hr = dev->DrawPrimitive(D3DPT_TRIANGLESTRIP, D3DFVF_TLVERTEX, rgverts, 4, 0);
//	fprintf(f, "DrawPrimitive returned %i. %s\n", hr, ErrorMsg(hr));
//	fflush(f);

	dev->EndScene();
}

HRESULT AlphaBlt(LPDIRECT3DDEVICE3 dev, LPDIRECTDRAWSURFACE4 dds, int x1, int y1, int x2, int y2,
			   float fade) {

//	FILE* f = fopen("alphablt.trace.txt", "wt");

	LPDIRECT3DTEXTURE2 texture;
	D3DTEXTUREHANDLE htex;

	D3DTLVERTEX rgverts[5] = {
		{ 0, 0, 0, 1.0f, 0x00000000, 0, 0.0f, 0.0f },
		{ 0, 0, 0, 1.0f, 0x00000000, 0, 0.0f, 0.0f },
		{ 0, 0, 0, 1.0f, 0x00000000, 0, 0.0f, 0.0f },
		{ 0, 0, 0, 1.0f, 0x00000000, 0, 0.0f, 0.0f },
		{ 0, 0, 0, 1.0f, 0x00000000, 0, 0.0f, 0.0f }
	};

	HRESULT hr;
	D3DCOLOR c = D3DRGBA(1, 1, 1, fade);

	rgverts[0].color = c;
	rgverts[0].sx = (D3DVALUE)x1;
	rgverts[0].sy = (D3DVALUE)y1;
	rgverts[0].sz = 1.0f;
	rgverts[0].specular = D3DRGBA(0, 0, 0, 0);
	rgverts[0].rhw = 1.0f;
	rgverts[0].tu = 0.0f;
	rgverts[0].tv = 0.0f;

	rgverts[1].color = c;
	rgverts[1].sx = (D3DVALUE)x2;
	rgverts[1].sy = (D3DVALUE)y1;
	rgverts[1].sz = 1.0f;
	rgverts[1].specular = D3DRGBA(0, 0, 0, 0);
	rgverts[1].rhw = 1.0f;
	rgverts[1].tu = 1.0f;
	rgverts[1].tv = 0.0f;

	rgverts[2].color = c;
	rgverts[2].sx = (D3DVALUE)x1;
	rgverts[2].sy = (D3DVALUE)y2;
	rgverts[2].sz = 1.0f;
	rgverts[2].specular = D3DRGBA(0, 0, 0, 0);
	rgverts[2].rhw = 1.0f;
	rgverts[2].tu = 0.0f;
	rgverts[2].tv = 1.0f;

   	rgverts[3].color = c;
	rgverts[3].sx = (D3DVALUE)x2;
	rgverts[3].sy = (D3DVALUE)y2;
	rgverts[3].sz = 1.0f;
	rgverts[3].specular = D3DRGBA(0, 0, 0, 0);
	rgverts[3].rhw = 1.0f;
	rgverts[3].tu = 1.0f;
	rgverts[3].tv = 1.0f;

	dev->SetRenderState(D3DRENDERSTATE_SRCBLEND,  D3DBLEND_SRCALPHA);
	dev->SetRenderState(D3DRENDERSTATE_DESTBLEND, D3DBLEND_INVSRCALPHA);
	dev->SetRenderState(D3DRENDERSTATE_ALPHABLENDENABLE, TRUE );
	dev->SetRenderState(D3DRENDERSTATE_SRCBLEND, D3DBLEND_BOTHSRCALPHA);

	DDSURFACEDESC2       ddsd;
	LPDIRECTDRAWSURFACE4 pddsTexture;
	LPDIRECT3DTEXTURE2  texBitmap, texTexture;
	
	ddsd.dwSize = sizeof(ddsd);
	hr = dds->GetSurfaceDesc(&ddsd);
	if(FAILED(hr)) return hr;
	ddsd.dwSize = sizeof(ddsd);
	ddsd.dwFlags = DDSD_CAPS | DDSD_HEIGHT | DDSD_WIDTH; // | DDSD_PIXELFORMAT;
	ddsd.ddsCaps.dwCaps = DDSCAPS_TEXTURE | DDSCAPS_ALLOCONLOAD | DDSCAPS_VIDEOMEMORY;

	hr = gpdd_directdraw->CreateSurface(&ddsd, &pddsTexture, NULL);
	if(FAILED(hr)) return hr;
//	const char *error = ErrorMsg(hr);
		
	hr = dds->QueryInterface(IID_IDirect3DTexture2, (LPVOID*)&texBitmap);
	if(FAILED(hr)) return hr;
	hr = pddsTexture->QueryInterface(IID_IDirect3DTexture2, (LPVOID*)&texTexture);
	if(FAILED(hr)) return hr;

	hr = texTexture->Load(texBitmap);
	if(FAILED(hr)) return hr;

//	hr = dev->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_BLENDTEXTUREALPHA);
	hr = dev->SetTexture(0, texture);
	if(FAILED(hr)) return hr;

	texBitmap->Release();
	
	// Draw the surface to the screen via DrawPrimitive.
	hr = dev->DrawPrimitive(D3DPT_TRIANGLESTRIP, D3DFVF_TLVERTEX, rgverts, 4, 0);
	if(FAILED(hr)) return hr;

	texTexture->Release();
	pddsTexture->Release();

//	fprintf(f, "DrawPrimitive returned %i. %s", hr, ErrorMsg(hr));
//	fflush(f);
}
