#include <dinput.h>
#include "global.h"
#include "input.h"
#include <string.h>
#include "snes9x/snes9x.h"
#include "dcflash.h"
#include "Menu.h"

#define CONTROLLER	1
#define KEYBOARD	2
#define INVALID		0

typedef struct {
	int axis;
	int buttons;
	int min;
	int max;
} joystickdata;

static LPDIRECTINPUT dinput = NULL;
static LPDIRECTINPUTDEVICE2 didevice[4] = { NULL, NULL, NULL, NULL };
static HANDLE hNewDeviceEvent = NULL;
static HANDLE hRemoveDeviceEvent = NULL;
static int devicetype[4];
static int ndidevice = 0;
joystickdata devdata[4];
static int lastdata[4];

BOOL CALLBACK DIEnumDevicesProc(LPCDIDEVICEINSTANCE lpddi, LPVOID pvRef);

void JoystickInit(void) {
#if defined USE_DIRECT_INPUT
	HRESULT hr;
	char str[256];

	ndidevice = 0;

#if defined UNDER_CE
	if(hNewDeviceEvent == NULL) {
		hNewDeviceEvent = CreateEvent(NULL, FALSE, FALSE, TEXT("MAPLE_NEW_DEVICE"));
		hRemoveDeviceEvent = CreateEvent(NULL, FALSE, FALSE, TEXT("MAPLE_DEVICE_REMOVED"));
		if (hNewDeviceEvent == NULL) {
			S9xMessage(0, 0, "Error Creating Device Event");
		} else {
#if defined DEBUG_INPUT
			S9xMessage(0, 0, "Device Event Created OK");
#endif
		}
		if(hRemoveDeviceEvent == NULL) {
			S9xMessage(0, 0, "Error Creating Remove Device Event");
		}
	}
#endif


	if(dinput == NULL) {
		hr = DirectInputCreate(gh_inst, DIRECTINPUT_VERSION, &dinput, NULL);
#if defined DEBUG_INPUT
		sprintf(str, "DirectInputCreate returned: %i. %s", hr, ErrorMsg(hr));
		S9xMessage(0, 0, str);
#endif
	}

	for(int i = 0; i < 4; i++) {
		if(didevice[i] != NULL) {
#if defined DEBUG_INPUT
			sprintf(str, "Releasing device %i. Ptr = %X", i, didevice[i]);
			S9xMessage(0, 0, str);
#endif
			didevice[i]->Release();
		}
	}

	dinput->EnumDevices(
		0,						// Enumerate all devices
		DIEnumDevicesProc,		// This is the callback function
		NULL,					// Application specific information (I use global variables)
		DIEDFL_ATTACHEDONLY);	// Only the devices currently attached

//	DWORD tick = GetTickCount();
//	while(GetTickCount() < tick + 2000);		// Wait
#endif
}

BOOL CALLBACK DIEnumDevicesProc(LPCDIDEVICEINSTANCE lpddi, LPVOID) {

/*#if defined DEBUG_INPUT
	// Get the device type and put it into a string buffer
	if(GET_DIDEVICE_TYPE(lpddi->dwDevType) == DIDEVTYPE_JOYSTICK) {
		superstrcpy(str2, TEXT("Controller"));
	} else if(GET_DIDEVICE_TYPE(lpddi->dwDevType) == DIDEVTYPE_KEYBOARD) {
		superstrcpy(str2, TEXT("Keyboard"));
	} else {
		superstrcpy(str2, TEXT("Other"));
	}

	// Print that I found a device
	sprintf(str, "Device found: %s", str2);
	S9xMessage(0, 0, str);
#endif */

	// Temporary device
	LPDIRECTINPUTDEVICE pdev;

	// Create the device
	HRESULT hr = dinput->CreateDevice(lpddi->guidInstance, &pdev, NULL);
	if(hr == DI_OK) {
		// If the device is good, Query for the IDirectInputDevice2 object
		hr = pdev->QueryInterface(IID_IDirectInputDevice2, 
			(LPVOID*)(&(didevice[ndidevice])));
	}

	// Again, if both of those calls are good, print that I successfully created the device
	if(hr == DI_OK) {
#if defined DEBUG_INPUT
		S9xMessage(0, 0, "Device created successfully");
#endif
		ndidevice++;
	} else {
		// If not, print a generic error message
		S9xMessage(0, 0, "Error creating device");
	}

	// Query the device type again (doing this twice keeps the DirectDraw code out)
	// And set some data about it
	if(GET_DIDEVICE_TYPE(lpddi->dwDevType) == DIDEVTYPE_JOYSTICK) {
		// This device is a controller of some sort

		// Set the data format to Joystick
		didevice[ndidevice-1]->SetDataFormat(&c_dfDIJoystick);
		// Set the global data devicetype[] to CONTROLLER (so I don't have to poll later)
		devicetype[ndidevice-1] = CONTROLLER;
		// Acquire the device (required by DirectInput)
		didevice[ndidevice-1]->Acquire();
		// Set the cooperate level (may be necessary)
		didevice[ndidevice-1]->SetCooperativeLevel(ghwd_window, DISCL_EXCLUSIVE | DISCL_FOREGROUND);

		// Query for Capabilities
		DIDEVCAPS dcaps;

		dcaps.dwSize = sizeof(dcaps);
		didevice[ndidevice-1]->GetCapabilities(&dcaps);
		devdata[ndidevice-1].axis = dcaps.dwAxes;
		devdata[ndidevice-1].buttons = dcaps.dwButtons;

		// Get some properties
		DIPROPRANGE diprop, dip2;

		diprop.diph.dwSize = sizeof(DIPROPRANGE);
		diprop.diph.dwHeaderSize = sizeof(DIPROPHEADER);
		diprop.diph.dwObj = 0;
		diprop.diph.dwHow = DIPH_DEVICE;
		diprop.lMax = 255;
		diprop.lMin = 0;
	
		didevice[ndidevice-1]->SetProperty(DIPROP_RANGE, &diprop.diph);

		devdata[ndidevice-1].max = diprop.lMax;
		devdata[ndidevice-1].min = diprop.lMin;

#if defined DEBUG_INPUT
		char str[256];
		sprintf(str, "Device had %i axis and %i buttons.", dcaps.dwAxes, dcaps.dwButtons);
		S9xMessage(0, 0, str);

		sprintf(str, "Device max - %i min - %i center - %i", 
			devdata[ndidevice-1].max,
			devdata[ndidevice-1].min,
			(devdata[ndidevice-1].max - devdata[ndidevice-1].min) / 2 + devdata[ndidevice-1].min);
		S9xMessage(0, 0, str);
#endif

	} else if(GET_DIDEVICE_TYPE(lpddi->dwDevType) == DIDEVTYPE_KEYBOARD) {
		// This device is a keyboard
		devicetype[ndidevice-1] = KEYBOARD;

		// Right now I only set the data format because its not actually read from
		didevice[ndidevice-1]->SetDataFormat(&c_dfDIKeyboard);
	} else {
		// This device isn't used
		devicetype[ndidevice-1] = INVALID;
	}
	
	// I only get 4 devices, thats how many the Dreamcast has
	if(ndidevice == 5) return false;
	return true;
}

void CheckForDevices(void) {
#if defined UNDER_CE
	if(WaitForSingleObject(hNewDeviceEvent, 0) != WAIT_TIMEOUT) {
#ifdef DEBUG_FLASH
		MessageBox(_T("Device Inserted"), false);
		MessageBox(_T("JoystickInit()"), false);
#endif
		JoystickInit();
#ifdef DEBUG_FLASH
		MessageBox(_T("FlashDeInit()"), false);
#endif
		FlashDeInit();
#ifdef DEBUG_FLASH
		MessageBox(_T("FlashInit()"), false);
#endif
		FlashInit();

//		ResetEvent(hNewDeviceEvent);
//		ResetEvent(hRemoveDeviceEvent);
	}
	if(WaitForSingleObject(hRemoveDeviceEvent, 0) != WAIT_TIMEOUT) {
//		MessageBox(_T("Device Removed"), false);
		FlashDeInit();
		FlashInit();

//		ResetEvent(hNewDeviceEvent);
//		ResetEvent(hRemoveDeviceEvent);
	}
#endif
}


// Called to poll an input device for data, called on a per-device basis
int GetInputDeviceData(int devicenum) {

	// If its a controller...
	if(devicetype[devicenum] == CONTROLLER) {
		DIJOYSTATE state;

		didevice[devicenum]->Poll();
		HRESULT hr = didevice[devicenum]->GetDeviceState(sizeof(DIJOYSTATE), &state);
		
		int st = 0;
		for(int i = 0; i < 12; i++) {
			int s = state.rgbButtons[i];
			int t = 0x80;
			int u = s ^ t;
			if(state.rgbButtons[i] | 0x80 == state.rgbButtons[i]) {
				st |= BUTTON[i];
			}
		}

		int x = (devdata[devicenum].max - devdata[devicenum].min) / 2 + devdata[devicenum].min;
		int d = 64;

		if(state.lX > x+d) st |= RIGHT;
		else if(state.lX < x-d) st |= LEFT;

		if(state.lY > x+d) st |= DOWN;
		else if(state.lY < x-d) st |= UP; 

		lastdata[devicenum] = st;
		return st;
	}

	return 0;

}
