#include "resource.h"
#include "Menu.h"
#include <d3d.h>
#include "global.h"
#include "Direct3D.h"
#include "input.h"
#include "Gui.h"

#ifdef DEBUG_MENU
#include "snes9x/snes9x.h"		// For S9xMessage
#endif

LPDIRECTDRAWSURFACE4 iconCART = NULL;
LPDIRECTDRAWSURFACE4 iconFOLDER = NULL;
LPDIRECTDRAWSURFACE4 iconRESET = NULL;
LPDIRECTDRAWSURFACE4 iconCONTROLLER = NULL;
LPDIRECTDRAWSURFACE4 iconVIDEO = NULL;
LPDIRECTDRAWSURFACE4 iconRETURN = NULL;
LPDIRECTDRAWSURFACE4 iconLOAD = NULL;
LPDIRECTDRAWSURFACE4 iconSAVE = NULL;

static HFONT fontLarge;
static HFONT fontSmall;

static LPDIRECTDRAWSURFACE4 surfAccept = NULL;
static LPDIRECTDRAWSURFACE4 surfCancel = NULL;
static LPDIRECTDRAWSURFACE4 surfEnd = NULL;
static LPDIRECTDRAWSURFACE4 surfUpArrow = NULL;
static LPDIRECTDRAWSURFACE4 surfDnArrow = NULL;

const int LEFT_OF_TEXT = 8;
const int LEFT_OF_ICON = 6;
const int ICONX = 40;
const int ICONY = 30;

void WaitForInputDone(void) {
	glpdsb_menuclick->Play(0, 0, 0);

	int a = 0;
	do {
		a = 0;
		for(int i = 0; i < 4; i++) {
			a += GetInputDeviceData(i);
		}
	} while(a);
}

void SetupMenu(void) {
#ifdef DEBUG_MENU
	S9xMessage(0, 0, "Setup Menu OK");
#endif
	LOGFONT lf;

	memset(&lf, 0, sizeof(lf));
	lf.lfHeight = FONTSIZE;
	lf.lfWeight = FW_BOLD;
	fontLarge = CreateFontIndirect(&lf);

	memset(&lf, 0, sizeof(lf));
	lf.lfHeight = SMALLFONTSIZE;
	//lf.lfWeight = FW_BOLD;
	fontSmall = CreateFontIndirect(&lf);

	surfAccept = LoadBitmap(IDB_ACCEPT);
	surfCancel = LoadBitmap(IDB_CANCEL);
	surfEnd = LoadBitmap(IDB_END);

	surfDnArrow = LoadBitmap(IDB_ARROW);
	surfUpArrow = LoadBitmap(IDB_UPARROW);

	iconCART = LoadBitmap(IDB_CART);
	iconFOLDER = LoadBitmap(IDB_FOLDER);
	iconRESET = LoadBitmap(IDB_RESET);
	iconCONTROLLER = LoadBitmap(IDB_CONTROLLER);
	iconVIDEO = LoadBitmap(IDB_VIDEO);
	iconRETURN = LoadBitmap(IDB_RETURN);
	iconLOAD = LoadBitmap(IDB_LOAD);
	iconSAVE = LoadBitmap(IDB_SAVE);

}

struct BOX {
	int x1, y1, x2, y2;
};

int DoMenu(int nText, const TCHAR* text[], int nOptions, OptionStruct option[],
		   LPDIRECTDRAWSURFACE4 background, int width, int height, int defselect,
		   bool allowcancel, bool allowinput) {
#ifdef USE_DIRECT_X

	int select = defselect;		// Number of selection (on screen)
	int start = 0;				// First option shown on screen (if scrolled)

	int uparrow = 0;			// 1 if up arrow is shown
	int dnarrow = 0;			// 1 if down arrow is shown

	int i, j;					// For loops
	HRESULT hr;					// For error information
	int x, y;					// Used for points

	int movedlast = -5;

	// Preparator Measurements
	BOX menu;

	char str[256];

	if(width == widthFIT) {
		HDC	dc;
		gdds_back->GetDC(&dc);
		SelectObject(dc, fontLarge);
		SetBkMode(dc, TRANSPARENT);

		int mcx = 0;
		for(i = 0; i < nText; i++) {
			SIZE sz;
			GetTextExtentPoint32(dc, text[i], _tcslen(text[i]), &sz);
			if(sz.cx > mcx) mcx = sz.cx;
		}
		for(i = 0; i < nOptions; i++) {
			SIZE sz;
			GetTextExtentPoint32(dc, option[i].text, _tcslen(option[i].text), &sz);
			if(sz.cx > mcx) mcx = sz.cx;
		}

		gdds_back->ReleaseDC(dc);

		width = mcx + FONTSIZE * 2;
		if(width > widthMAX) width = widthMAX;
	}
	if(height == heightFIT) {
		height = nText + nOptions;
		if(height > heightMAX) height = heightMAX;
	}

	if(height < nText + nOptions) {
		if(select > height - nText - 1) {
			start = select;
			select = 0;

			int wubba = start - nOptions + (height - nText - 1);
			if(wubba > 0) {
				start -= wubba;
				select += wubba;
			}
		}
	}


	menu.x1 = SCREENX / 2 - width / 2;
	menu.x2 = menu.x1 + width;
	menu.y1 = SCREENY / 2 - (height * FONTSIZE)  / 2;
	menu.y2 = menu.y1 + (height * FONTSIZE);
		
	// Main menu loop
	while(1) {
		uparrow = 0; dnarrow = 0;
		if(height < nText + nOptions) dnarrow = 1;

		// Direct3D calls
		gdev_device->BeginScene();
//		AlphaBlt(gdev_device, background, 0, 0, 640, 480, 1.0);
		FillGradientRect(gdev_device, 0,0,640,480,0xFFFFFFFF, D3DRGBA(0.5,0.5,0.5,1), D3DRGBA(0.5,0.5,0.5,1),
			D3DRGBA(0,0,0,1));

		// Draw the menu box
		FillGradientRect(gdev_device, menu.x1, menu.y1, menu.x2, menu.y2,
			D3DRGBA(0.0, 0.2, 1.0, 0.5), D3DRGBA(0.0, 0.2, 1.0, 0.5), 
			D3DRGBA(0.0, 0.2, 1.0, 0.5), D3DRGBA(0.0, 0.2, 1.0, 0.5));

		// Draw the selection band
		if(nOptions != 0) {
			FillGradientRect(gdev_device, menu.x1, menu.y1 + (select + nText) * FONTSIZE,
				menu.x2, menu.y1 + (select + nText) * FONTSIZE + FONTSIZE,
				D3DRGBA(0.0, 0.3, 1.0, 0.8), D3DRGBA(0.0, 0.3, 1.0, 0.8),
				D3DRGBA(0.0, 0.0, 0.3, 0.8), D3DRGBA(0.0, 0.0, 0.3, 0.8));
		}

		gdev_device->EndScene();

		// Draw accept/cancel
		if(allowinput) {
		if(allowcancel) {
			x = SCREENX/2 - 90;
			y = SCREENY - 32 - 14;
			RECT r1 = { x, y, x + 90, y + 27 };
			gdds_back->Blt(&r1, surfAccept, NULL, DDBLT_WAIT, NULL);
			RECT r3 = { 320, y, 320 + 90, y + 27 };
			gdds_back->Blt(&r3, surfCancel, NULL, DDBLT_WAIT, NULL);
		} else {
			x = 640/2 - 46;
			y = 480 - 32 - 14;
			RECT r1 = { x, y, x + 90, y + 27 };
			gdds_back->Blt(&r1, surfAccept, NULL, DDBLT_WAIT, NULL);
			RECT r3 = { x+90, y, x+90+2, y + 27 };
			gdds_back->Blt(&r3, surfEnd, NULL, DDBLT_WAIT, NULL);
		}
		}

		// Draw the arrows
		x = SCREENX / 2 - 11;
		y = menu.y2 - FONTSIZE/2 - 10;
		if(dnarrow && (nOptions + 1 > start + (height - nText))) {
			RECT r = { x, y, x + 22, y + 20 };
			gdds_back->Blt(&r, surfDnArrow, NULL, DDBLT_WAIT, NULL);
		}
		y = menu.y1 + FONTSIZE/2 - 10;
		if(start != 0) {
			RECT r2 = { x, y,  x + 22, y + 20 };
			gdds_back->Blt(&r2, surfUpArrow, NULL, DDBLT_WAIT, NULL);
			uparrow = 1;
		} 

		// Draw the icons
		x = menu.x1 + LEFT_OF_ICON;
		y = menu.y1 + nText * FONTSIZE;
		RECT rect = { x, y, x + ICONX, y + ICONY };
		for(i = start; i < start + height - nText - dnarrow; i++) {
			if(option[i].icon != NULL) {
				gdds_back->Blt(&rect, option[i].icon, NULL, DDBLT_WAIT, NULL);
			}
			rect.top += FONTSIZE;
			rect.bottom += FONTSIZE;
		} 

		// Draw the text
		HDC	dc;
		gdds_back->GetDC(&dc);
		
		SelectObject(dc, fontLarge);
		SetBkMode(dc, TRANSPARENT);
		if(nOptions == 0) SetTextColor(dc, RGB(255, 255, 255));
		else SetTextColor(dc, RGB(0, 0, 0));

		if(start == 0) {
		for(i = 0; i < nText; i++) {
			SIZE sz;
			GetTextExtentPoint32(dc, text[i], _tcslen(text[i]), &sz);
			int x = SCREENX / 2 - sz.cx / 2;
			ExtTextOut(dc, x, menu.y1 + (i * FONTSIZE), NULL, NULL, 
				text[i], _tcslen(text[i]), NULL);
		}
		}

		SetTextColor(dc, RGB(255, 255, 255));
		int y = menu.y1 + nText * FONTSIZE;
		for(i = start; i < start + height - nText - dnarrow; i++) {
			int x = menu.x1 + LEFT_OF_TEXT;
			if(option[i].align == alignCENTER) {
				SIZE sz;
				GetTextExtentPoint32(dc, option[i].text, _tcslen(option[i].text), &sz);
				x = SCREENX / 2 - sz.cx / 2;
			} else if(option[i].icon != NULL) {
				x += ICONX + LEFT_OF_ICON;
			}
			ExtTextOut(dc, x, y, NULL, NULL, 
				option[i].text, _tcslen(option[i].text), NULL);
			y += FONTSIZE;
		} 
		
		gdds_back->ReleaseDC(dc);

		FlipScreen();

		if(!allowinput) return 0;

		MSG msg;

		// Allow Windows messages to get through - most specifically, ALT+F4
		if(PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE)) {
			if(!GetMessage(&msg, NULL, 0, 0))
				return 0;
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}

		CheckForDevices();
		
		// Accept controller input
		if(!movedlast) {
			for(int d = 0; d < 4; d++) {
				int a = GetInputDeviceData(d);
				if((a | UP) == a) {
					select--;
					if(select < 0 && start != 0) { start--; }
					if(select < 0) select = 0;
					break;
				}
				if((a | DOWN) == a) {
					select++;
					if(select + start > nOptions - 1) { 
						select--; 
					}
					if((select >= height - nText - 1) && dnarrow) {
						select = height - nText - 2;
						start++;
					}
					break;
				}
				if((a | BUTTON[3]) == a) {
					select--;
					if(select < 0 && start != 0) { start--; }
					if(select < 0) select = 0;
					break;
				}
				if((a | BUTTON[4]) == a) {
					select++;
					if(select + start > nOptions - 1) { 
						select--; 
					}
					if((select >= height - nText - 1) && dnarrow) {
						select = height - nText - 2;
						start++;
					}
					break;
				}
				if((a | BUTTON[1]) == a) {
					WaitForInputDone();
					return select + start;
				}
				if(((a | BUTTON[0]) == a) && allowcancel) {
					WaitForInputDone();
					return -1;
				}
				if(a) break;
			}
			movedlast++;
#ifndef UNDER_CE
			movedlast = 0;
#endif
		} else {
			if(movedlast < 1) movedlast++;
			else movedlast = 0;
		} 
	}

#else
	return 0;
#endif
}

void MessageBox(const TCHAR* message, bool allowinput) {

	const TCHAR* txt[] = { _T(" "), message, _T(" ") };
	DoMenu(3, txt, 0, NULL, NULL, widthFIT, heightFIT, 0, false, allowinput);

}