#include "global.h"
#if defined DO_SOUND

#include "Sound.h"

static LPDIRECTSOUND lpDirectSound;
static LPDIRECTSOUNDBUFFER lpDirectSoundBuffer;

// DEBUG STUFF

#include <stdio.h>
#include "Menu.h"

inline bool ERROR_MSG(TCHAR *str, HRESULT hr) {
#if defined DEBUG_SOUND
	MessageBox(str);
	MessageBox(ErrorMsg(hr));
	if(FAILED(hr)) return true;
	return false;
#else
	return false;
#endif
}

inline void GET_LAST_ERROR(TCHAR *str) {
#if defined DEBUG_SOUND
	TCHAR str2[256];
	_stprintf(str2, _T("GetLastError() = %i"), GetLastError());
	MessageBox(str);
	MessageBox(str2);
#endif
}

// STATIC FUNCTIONS

// ParseWaveFile
// Copied from DSUTIL.CPP
// Copyright (c) 1996, 1997 Microsoft Corporation
bool ParseWaveFile(void *pvWaveFile, WAVEFORMATEX **ppWaveHeader,
				   BYTE **ppbWaveData, DWORD *pcbWaveSize) {
    DWORD *pdw;
    DWORD *pdwEnd;
    DWORD dwRiff;
    DWORD dwType;
    DWORD dwLength;

    if (ppWaveHeader)
        *ppWaveHeader = NULL;

    if (ppbWaveData)
        *ppbWaveData = NULL;

    if (pcbWaveSize)
        *pcbWaveSize = 0;

    pdw = (DWORD *)pvWaveFile;
    dwRiff   = *pdw++;
    dwLength = *pdw++;
    dwType   = *pdw++;

    // Check if it's a WAV format file
    if (dwType != mmioFOURCC('W', 'A', 'V', 'E'))
        return FALSE;

    // Check if it's a RIFF format file
    if (dwRiff != mmioFOURCC('R', 'I', 'F', 'F'))
        return FALSE;

    pdwEnd = (DWORD *)((BYTE *)pdw + dwLength-4);

    while (pdw < pdwEnd)
    {
        dwType = *pdw++;
        dwLength = *pdw++;

        switch (dwType)
        {
        case mmioFOURCC('f', 'm', 't', ' '):
            if (ppWaveHeader && !*ppWaveHeader)
            {
                if (dwLength < sizeof(WAVEFORMAT))
                    return FALSE;

                *ppWaveHeader = (WAVEFORMATEX *)pdw;

                if ((!ppbWaveData || *ppbWaveData) && (!pcbWaveSize || *pcbWaveSize))
                    return TRUE;
            }
            break;

        case mmioFOURCC('d', 'a', 't', 'a'):
            if ((ppbWaveData && !*ppbWaveData) || (pcbWaveSize && !*pcbWaveSize))
            {
                if (ppbWaveData)
                    *ppbWaveData = (LPBYTE)pdw;

                if (pcbWaveSize)
                    *pcbWaveSize = dwLength;

                if (!ppWaveHeader || *ppWaveHeader)
                    return TRUE;
            }
            break;
        }

        pdw = (DWORD *)((BYTE *)pdw + ((dwLength+1)&~1));
    }

    return FALSE;
}

// GetWaveResource
// Copied from DSUTIL.CPP
// Copyright (c) 1996, 1997 Microsoft Corporation
// Modified August 17, 2001 by Jamin Vander Berg for Sintendo
bool GetWaveResource(LPCTSTR tszName, WAVEFORMATEX **ppWaveHeader,
					 BYTE **ppbWaveData, DWORD *pcbWaveSize) {

    HRSRC   hResInfo;
    HGLOBAL hResData;
    void    *pvRes;
    HANDLE  hFile;
    BYTE    *rgbyFileTemp;
    unsigned long cbRead;
    DWORD   dwSize;

    // Find the specifed WAV resource
    hResInfo = FindResource(gh_inst, tszName, TEXT("WAVE"));
    if (hResInfo == NULL) {
#if defined DEBUG_SOUND
		GET_LAST_ERROR(_T("FindResource Failed"));
#endif
        goto TryFile;
	}

    // Load the Resource
    hResData = LoadResource(gh_inst, hResInfo);
    if (hResData == NULL) {
#if defined DEBUG_SOUND
		GET_LAST_ERROR(_T("LoadResource() Failed"));
#endif
        goto TryFile;
	}

    // Lock the Resource
    pvRes = LockResource(hResData);
    if (pvRes == NULL) {
#if defined DEBUG_SOUND
		GET_LAST_ERROR(_T("LockResource() Failed"));
#endif
        goto TryFile;
	}

    // Read and parse the Resource
    if (ParseWaveFile(pvRes, ppWaveHeader, ppbWaveData, pcbWaveSize) != NULL)
        return TRUE;
	else {
#if defined DEBUG_SOUND
		MessageBox(_T("ParseWaveFile() Failed"));
#endif
	}

TryFile:

    // Failed to load as a resource - try it as a file
    hFile = CreateFile(tszName, GENERIC_READ, 0, NULL, OPEN_EXISTING, 0, NULL);
    if (hFile == INVALID_HANDLE_VALUE)
        return FALSE;

    dwSize = GetFileSize(hFile, NULL);

    rgbyFileTemp = (BYTE*) malloc(dwSize);
    if (!rgbyFileTemp)
        return FALSE;

    ReadFile (hFile, rgbyFileTemp, dwSize, &cbRead, NULL);
    CloseHandle(hFile);
    if (cbRead != dwSize)
        return FALSE;

    if (ParseWaveFile(rgbyFileTemp, ppWaveHeader, ppbWaveData, pcbWaveSize) == NULL)
        return FALSE;

    return TRUE;
}

// FillSoundBuffer
// Copied from DSUTIL.CPP
// Copyright (c) 1996, 1997 Microsoft Corporation
// Modified August 17, 2001 by Jamin Vander Berg for Sintendo
bool FillSoundBuffer(IDirectSoundBuffer *pdsb, BYTE *pbWaveData, DWORD dwWaveSize) {
    LPVOID pMem1, pMem2;
    DWORD  dwSize1, dwSize2;
	HRESULT hr;

    if (!pdsb || !pbWaveData || !dwWaveSize)
        return false;

	hr = pdsb->Lock(0, dwWaveSize, &pMem1, &dwSize1, &pMem2, &dwSize2, 0);
    ERROR_MSG(TEXT("Lock SoundBuffer"), hr);
	if(FAILED(hr)) return false;

    memcpy(pMem1, pbWaveData, dwSize1);

    if (dwSize2 != 0)
        memcpy(pMem2, pbWaveData+dwSize1, dwSize2);

    pdsb->Unlock(pMem1, dwSize1, pMem2, dwSize2);

    return true;
}

// EXTERNAL FUNCTIONS

HRESULT InitDirectSound(void) {
	HRESULT       hr;
	DSCAPS dscaps; 
	
	hr = DirectSoundCreate(NULL, &lpDirectSound, NULL);
	ERROR_MSG(_T("DirectSoundCreate()"), hr);
	if(FAILED(hr)) return hr;

	hr = lpDirectSound->SetCooperativeLevel(ghwd_window, DSSCL_EXCLUSIVE);
	ERROR_MSG(_T("SetCooperateLevel()"), hr);
	if(FAILED(hr)) return hr;

	memset(&dscaps, 0, sizeof(DSCAPS));
	dscaps.dwSize = sizeof(DSCAPS); 
	hr = lpDirectSound->GetCaps(&dscaps); 
	ERROR_MSG(_T("GetCaps()"), hr);
	if(FAILED(hr)) return hr;

#if defined DEBUG_SOUND
	TCHAR str[256];

	_stprintf(str, _T("Total Hardware Memory: %i"), dscaps.dwTotalHwMemBytes);
	MessageBox(str);
	_stprintf(str, _T("Free Hardware Memory: %i"), dscaps.dwFreeHwMemBytes);
	MessageBox(str);
#endif

    // Set up wave format structure. 
    PCMWAVEFORMAT pcmwf; 

    memset(&pcmwf, 0, sizeof(PCMWAVEFORMAT)); 
    pcmwf.wf.wFormatTag = WAVE_FORMAT_PCM; 
    pcmwf.wf.nChannels = 2; 
    pcmwf.wf.nSamplesPerSec = 44100; 
    pcmwf.wf.nBlockAlign = 4; 
    pcmwf.wf.nAvgBytesPerSec = pcmwf.wf.nSamplesPerSec * pcmwf.wf.nBlockAlign; 
    pcmwf.wBitsPerSample = 16; 
    
	// Set up DSBUFFERDESC structure. 
    DSBUFFERDESC dsbdesc; 

    memset(&dsbdesc, 0, sizeof(DSBUFFERDESC)); // Zero it out. 
    dsbdesc.dwSize = sizeof(DSBUFFERDESC); 
    dsbdesc.dwFlags = DSBCAPS_CTRLVOLUME;
    dsbdesc.dwBufferBytes = 3 * pcmwf.wf.nAvgBytesPerSec;   // 3-second buffer 
    dsbdesc.lpwfxFormat = (LPWAVEFORMATEX)&pcmwf; 

    // Create buffer. 
    hr = lpDirectSound->CreateSoundBuffer(&dsbdesc, &lpDirectSoundBuffer, NULL); 
    
	ERROR_MSG(_T("CreateSoundBuffer()"), hr);
	if(FAILED(hr)) return hr;

	int nBytes = dsbdesc.dwBufferBytes;
	
    LPVOID lpvPtr1; 
    DWORD dwBytes1; 
    LPVOID lpvPtr2; 
    DWORD dwBytes2; 

	hr = lpDirectSoundBuffer->Lock(0, nBytes, &lpvPtr1, &dwBytes1, &lpvPtr2, &dwBytes2, 0); 
	ERROR_MSG(_T("Lock()"), hr);
	if(FAILED(hr)) return hr;

	for(int i = 0; i < nBytes/4; i++) {
		((unsigned int*)lpvPtr1)[i] = 0xFFFFFFFF;
	}

	hr = lpDirectSoundBuffer->Unlock(lpvPtr1, dwBytes1, lpvPtr2, dwBytes2); 
	ERROR_MSG(_T("Unlock()"), hr);
	if(FAILED(hr)) return hr;

	lpDirectSoundBuffer->SetVolume(DSBVOLUME_MAX);
	lpDirectSoundBuffer->SetCurrentPosition(0);

	hr = lpDirectSoundBuffer->Play(0, 0, 0);
	ERROR_MSG(_T("Play()"), hr);
	if(FAILED(hr)) return hr;
}

// LoadSoundBuffer
// Copied from DSUTIL.CPP
// Copyright (c) 1996, 1997 Microsoft Corporation
// Modified August 17, 2001 by Jamin Vander Berg for Sintendo
LPDIRECTSOUNDBUFFER LoadSoundBuffer(TCHAR* name) {
    IDirectSoundBuffer *pdsb = NULL;
    DSBUFFERDESC dsbd = {0};
    BYTE *pbWaveData;

    if (GetWaveResource(name, &dsbd.lpwfxFormat, &pbWaveData, &dsbd.dwBufferBytes))
    {
        dsbd.dwSize = sizeof(dsbd);
        dsbd.dwFlags = DSBCAPS_CTRLVOLUME;

        if (SUCCEEDED(lpDirectSound->CreateSoundBuffer(&dsbd, &pdsb, NULL)))
        {
            if (!FillSoundBuffer(pdsb, pbWaveData, dsbd.dwBufferBytes))
            {
                MessageBox(_T("FillSoundBuffer() Failed"));
				pdsb->Release();
                pdsb = NULL;
            }
        } else {
			MessageBox(_T("CreateSoundBuffer() Failed"));
            pdsb = NULL;
		}
    } else {
		MessageBox(_T("GetWaveResource() Failed"));
	}

    return pdsb;
}

#endif